import logging
import tomllib

import discord

from botbotbot.ai import AIBot
from botbotbot.nicks import NickBot
from botbotbot.shuffle import Shuffler
from botbotbot.text import TextBot
from botbotbot.tts import CambAI
from botbotbot.voice import VoiceBot
from botbotbot.wordlist import Wordlist


def main() -> None:
    description = """Discord Chaos Bot"""

    logger = logging.getLogger(__name__)
    logging.basicConfig(level=logging.INFO)

    with open("config.toml", "rb") as config_file:
        config = tomllib.load(config_file)

    guild_ids = config.get("guild_ids")
    if not (isinstance(guild_ids, list) and all(isinstance(i, int) for i in guild_ids)):
        logger.error("Guild IDs must be a list of integers.")
        guild_ids = []

    system_prompt = """Tu es une intelligence artificelle qui répond en français.
    Tu dois faire un commentaire pertinent en lien avec ce qui te sera dit.
    Ta réponse doit être très courte.
    Ta réponse doit être une seule phrase.
    TA RÉPONSE DOIT ÊTRE EN FRANÇAIS !!!"""

    aibot: AIBot | None = None

    if isinstance(key := config.get("mistral_api_key"), str):
        aibot = AIBot(
            key,
            model="mistral-large-latest",
            system_message=system_prompt,
        )
    else:
        logger.warning("No AI Bot.")

    cambai: CambAI | None = None
    if isinstance(key := config.get("cambai_api_key"), str):
        cambai = CambAI(key)
    else:
        logger.warning("No CambAI.")

    intents = discord.Intents.default()
    intents.members = True
    intents.message_content = True
    intents.reactions = True
    intents.voice_states = True

    bot = discord.Bot(description=description, intents=intents)
    shf = Shuffler()
    wl = Wordlist(bot, guild_ids)
    wl.init_events()

    text_bot = TextBot(bot, wl, aibot=aibot, shuffler=shf, guild_ids=guild_ids)
    text_bot.init_events()

    voice_bot = VoiceBot(bot, cambai, aibot=aibot, shuffler=shf, guild_ids=guild_ids)
    voice_bot.init_events()

    nick_bot = NickBot(bot, shuffler=shf, guild_ids=guild_ids)
    nick_bot.init_events()

    @bot.listen("on_ready")
    async def on_ready() -> None:
        logger.info(f"We have logged in as {bot.user}")

    bot.run(config.get("token"))
